unit GWDropDemo_Main;

{---------------------------------------------------------
DropTest: Test/demo application for OLE Drop mechanism.

Author: Graham Wideman

Revisions
---------

98-06-20 GW Original

----------------------------------------------------------}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls,
  GWDropTarget, DropDemoAppInterface, Grids;

type
  TIDT_DisplayInfo = record
    Lb:  TLabel;
    Sh: TShape;
  end;

  Tfm_Main = class(TForm)
    MainMenu1: TMainMenu;
    m_File: TMenuItem;
    mi_Quit: TMenuItem;
    m_Help: TMenuItem;
    mi_About: TMenuItem;
    tm_Update: TTimer;
    gb_IDropTarget: TGroupBox;
    gb_IEnumFormat: TGroupBox;
    sg_DragDiag: TStringGrid;
    GroupBox1: TGroupBox;
    Image1: TImage;
    sh_IDT_Regd: TShape;
    lb_IDT_Regd: TLabel;
    sh_IDT_Locked: TShape;
    lb_IDT_Locked: TLabel;
    sh_IDT_Exists: TShape;
    lb_IDT_Exists: TLabel;
    sh_IDT_Start: TShape;
    lb_IDT_Start: TLabel;
    GroupBox2: TGroupBox;
    bt_ClearCallCounts: TButton;
    StaticText1: TStaticText;
    sh_update: TShape;
    rg_Drop: TRadioGroup;
    GroupBox3: TGroupBox;
    bt_AddRef: TButton;
    bt_Release: TButton;
    rg_IDropTarget: TRadioGroup;
    sg_EnumFormatEtc: TStringGrid;
    gb_Data: TGroupBox;
    mo_Data: TMemo;
    stx_DropHere: TStaticText;
    procedure mi_AboutClick(Sender: TObject);
    procedure mi_QuitClick(Sender: TObject);
    procedure tm_UpdateTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bt_IDT_StateButtons(Sender: TObject);
    procedure bt_ClearCallCountsClick(Sender: TObject);
    procedure rg_DropClick(Sender: TObject);
    procedure bt_AddRefClick(Sender: TObject);
    procedure bt_ReleaseClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure rg_IDropTargetClick(Sender: TObject);
    procedure sg_EnumFormatEtcDblClick(Sender: TObject);
  private
    { Private declarations }
    FDropAppInterface: TDropDemoAppInterface;
    FIDT_DisplayInfo: array[gwdt_ls_start..gwdt_ls_regd] of TIDT_DisplayInfo;
    FIDT_DisplayState: Tgwdt_ls;
    FTestIRef: IUnknown;

    procedure LayoutGrids;
    procedure IDT_Display_Init;
    procedure IDT_LifeStateDisplayUpdate;
    procedure StgMediumToMemo(FmtNum: integer);
  public
    { Public declarations }
    procedure OnOLEDrop;
  end;

var
  fm_Main: Tfm_Main;

implementation

uses ActiveX, fm_About;

{$R *.DFM}

//-------------------------------------------------
procedure Tfm_Main.FormCreate(Sender: TObject);
//-------------------------------------------------
begin
  // Init other objects
  FTestIRef := nil;
  LayoutGrids;

  FDropAppInterface := TDropDemoAppInterface.Create(Self);
  FDropAppInterface.Mon_Enable := true;
  FDropAppInterface.Mon_GridInit(sg_DragDiag,1);

  // Init our display
  Top := 100;
  Left := 100;

  IDT_Display_Init;
end;

//-------------------------------------------------
procedure Tfm_Main.FormDestroy(Sender: TObject);
//-------------------------------------------------
begin
  FTestIRef := nil;
  FDropAppInterface.Destroy;
end;

//-------------------------------------------------
procedure Tfm_Main.IDT_Display_Init;
//-------------------------------------------------
Begin
  FIDT_DisplayInfo[gwdt_ls_Start ].Lb := lb_IDT_Start  ;
  FIDT_DisplayInfo[gwdt_ls_Exists].Lb := lb_IDT_Exists ;
  FIDT_DisplayInfo[gwdt_ls_Locked].Lb := lb_IDT_Locked ;
  FIDT_DisplayInfo[gwdt_ls_Regd  ].Lb := lb_IDT_Regd   ;

  FIDT_DisplayInfo[gwdt_ls_Start ].Sh := sh_IDT_Start  ;
  FIDT_DisplayInfo[gwdt_ls_Exists].Sh := sh_IDT_Exists ;
  FIDT_DisplayInfo[gwdt_ls_Locked].Sh := sh_IDT_Locked ;
  FIDT_DisplayInfo[gwdt_ls_Regd  ].Sh := sh_IDT_Regd   ;

  FIDT_DisplayState := gwdt_ls_exists; // bogus value to force display update
  IDT_LifeStateDisplayUpdate;
end;

//-------------------------------------------------
procedure Tfm_Main.IDT_LifeStateDisplayUpdate;
//-------------------------------------------------
Var
  N: Tgwdt_ls;
  cl: TColor;
Begin
  If FIDT_DisplayState <> FDropAppInterface.DropTarget_LifeState then
  Begin
    For N := gwdt_ls_start to gwdt_ls_regd do
    Begin
      If N = FDropAppInterface.DropTarget_LifeState then cl := clLime else cl := clWhite;
      FIDT_DisplayInfo[N].Sh.Brush.Color := cl;
      FIDT_DisplayInfo[N].Lb.Color := cl;

      FIDT_DisplayInfo[N].Sh.Update;
      FIDT_DisplayInfo[N].Lb.Update;
    end;
    FIDT_DisplayState := FDropAppInterface.DropTarget_LifeState;
  end;
end;

//-------------------------------------------------
procedure Tfm_Main.mi_AboutClick(Sender: TObject);
//-------------------------------------------------
begin
  fm_AboutBox.Left := Left+ 100;
  fm_AboutBox.Top  := Top + 100;
  fm_AboutBox.ShowModal;
end;

//-------------------------------------------------
procedure Tfm_Main.mi_QuitClick(Sender: TObject);
//-------------------------------------------------
begin
  Application.Terminate;
end;

//-------------------------------------------------
procedure Tfm_Main.tm_UpdateTimer(Sender: TObject);
//-------------------------------------------------
begin
  // Show that we're awake and getting attention
  if sh_Update.Brush.Color = clLime then
    sh_Update.Brush.Color := clBlack else
    sh_Update.Brush.Color := clLime;

  sh_Update.Update;  // force repaint

  // Update counts
  If Assigned(FDropAppInterface) then
  Begin
    FDropAppInterface.Mon_GridUpdate(sg_DragDiag,1);
    sg_DragDiag.Update;
  end;

  // Update State display if needed
  IDT_LifeStateDisplayUpdate;
end;

//-------------------------------------------------
procedure Tfm_Main.bt_IDT_StateButtons(Sender: TObject);
//-------------------------------------------------
Var
  NewState: Tgwdt_ls;
  Rslt: HResult;
Label error_end, good_end;
begin
  NewState := gwdt_ls_Start;

  If Sender = lb_IDT_Start  then NewState := gwdt_ls_Start;
  If Sender = lb_IDT_Exists then NewState := gwdt_ls_Exists;
  If Sender = lb_IDT_Locked then NewState := gwdt_ls_Locked;
  If Sender = lb_IDT_Regd   then NewState := gwdt_ls_Regd;

  Rslt := S_OK;

  If (NewState > gwdt_ls_Start) and
     (not FDropAppInterface.DropTarget_Exists) then
  Begin { create DropTarget to start with }
    Rslt := FDropAppInterface.DropTarget_Create(stx_DropHere, false);
    If Rslt <> S_OK then goto error_end;
  end;

  If NewState <> FDropAppInterface.DropTarget_LifeState then
  Begin
    Case NewState of
      gwdt_ls_Start  : Rslt := FDropAppInterface.DropTarget.ToState_Exists; // see below
      gwdt_ls_Exists : Rslt := FDropAppInterface.DropTarget.ToState_Exists;
      gwdt_ls_Locked : Rslt := FDropAppInterface.DropTarget.ToState_Locked;
      gwdt_ls_Regd   : Rslt := FDropAppInterface.DropTarget.ToState_Regd;
    end; { case }
    If Rslt <> S_OK then goto error_end;
  end;

  {-----------------------------------------------------------
   Special case... if no refs were ever created for object, then
   object never goes through RefCount 1 --> 0 transition, so
   system will not destroy it.  We have to destroy "manually" if
   we want to get rid of it.
  ------------------------------------------------------------}
  If (NewState = gwdt_ls_Start) and
     (FDropAppInterface.DropTarget_Exists) then
  Begin
    FDropAppInterface.DropTarget.Destroy;
  end;

  goto good_end;

error_end:
  Beep;

good_end:
  IDT_LifeStateDisplayUpdate;
end;


//-------------------------------------------------
procedure Tfm_Main.bt_ClearCallCountsClick(Sender: TObject);
//-------------------------------------------------
begin
  FDropAppInterface.Mon_ClearCallCounts;
end;

//-------------------------------------------------
procedure Tfm_Main.rg_DropClick(Sender: TObject);
//-------------------------------------------------
begin
  If FDropAppInterface.DropTarget_Exists then
  Begin
    If rg_Drop.ItemIndex = 1 then
      FDropAppInterface.DropTarget.Enable := true else
      FDropAppInterface.DropTarget.Enable := false;
  end else
  Begin
    rg_Drop.ItemIndex := 1; { that's default on Create }
  end;
end;

//-------------------------------------------------
procedure Tfm_Main.bt_AddRefClick(Sender: TObject);
//-------------------------------------------------
begin
  If FDropAppInterface.DropTarget_Exists then
  Begin
    FDropAppInterface.DropTarget.TestAddRef;
  end;
end;

//-------------------------------------------------
procedure Tfm_Main.bt_ReleaseClick(Sender: TObject);
//-------------------------------------------------
begin
  If FDropAppInterface.DropTarget_Exists then
  Begin
    FDropAppInterface.DropTarget.TestRelease;
  end;
end;

//-------------------------------------------------
procedure Tfm_Main.rg_IDropTargetClick(Sender: TObject);
//-------------------------------------------------
begin
  If rg_IDropTarget.ItemIndex = 1 then
  Begin
    if FDropAppInterface.DropTarget_Exists then
    Begin
      FTestIRef := FDropAppInterface.DropTarget;
    end else Beep;
  end else
  Begin
    FTestIRef := nil;
  end;
  If Assigned(FTestIRef) then
    rg_IDropTarget.ItemIndex := 1 else
    rg_IDropTarget.ItemIndex := 0;
end;

//-------------------------------------------------
procedure Tfm_Main.StgMediumToMemo(FmtNum: integer);
//-------------------------------------------------
Begin
  mo_Data.Lines.Clear;
  FDropAppInterface.Data_ContentToLines(FmtNum, mo_Data.Lines, 4096);
end;

//-------------------------------------------------
procedure Tfm_Main.sg_EnumFormatEtcDblClick(Sender: TObject);
//-------------------------------------------------
Var
  SelRow: integer;
  FmtNum: integer;
  S: string;
begin
  SelRow := sg_EnumFormatEtc.Selection.Top;
  FmtNum := SelRow - 2;
  StgMediumToMemo(FmtNum);
end;


{-----------------------------------------------
Grid layout info
-----------------------------------------------}
Type
  TGridInfo = record
    TotalColCount : integer;
    FixedCols     : integer;
    FixedRows     : integer;
  end;

  TGridColInfo = record
    H0: string;
    H1: string;
    Width: integer;
  end;

Const
  gi_DragDiag: TGridInfo =
    (TotalColCount : 3; FixedCols : 1; FixedRows : 1);

  gci_DragDiag: array[0..2] of TGridColInfo = (
    (H0: 'Item'   ; H1: ''   ; Width: 64),
    (H0: 'Value'  ; H1: ''   ; Width: 48),
    (H0: 'String' ; H1: ''   ; Width: 48)
  );

  gi_EnumFormatEtc: TGridInfo =
    (TotalColCount : 8; FixedCols : 1; FixedRows : 2);

  gci_EnumFormatEtc: array[0..7] of TGridColInfo = (
    (H0: ''        ; H1: 'Row'        ; Width: 32),
    (H0: 'cfFormat'; H1: 'number'     ; Width: 48),
    (H0: 'cfFormat'; H1: 'name'       ; Width: 128),
    (H0: 'ptd'     ; H1: 'for device' ; Width: 64),
    (H0: 'dwAspect'; H1: 'coarseness' ; Width: 64),
    (H0: 'lindex'  ; H1: 'pg layout'  ; Width: 64),
    (H0: 'tymed'   ; H1: 'location'   ; Width: 64),
    (H0: 'HGLOBAL' ; H1: 'datasize'   ; Width: 64)
  );

//-------------------------------------------------
procedure Tfm_Main.LayoutGrids;
//-------------------------------------------------
Var
  ColNum: integer;
begin
  //-----------------------------------------------------------
  sg_DragDiag.RowCount  := gi_DragDiag.FixedRows + 1;
  sg_DragDiag.FixedRows := gi_DragDiag.FixedRows;
  sg_DragDiag.ColCount  := gi_DragDiag.TotalColCount;
  sg_DragDiag.FixedCols := gi_DragDiag.FixedCols;

  For ColNum := 0 to sg_DragDiag.ColCount - 1 do
  Begin
    sg_DragDiag.ColWidths[ColNum] := gci_DragDiag[ColNum].Width;
    sg_DragDiag.Cells[ColNum,0] := gci_DragDiag[ColNum].H0;
  end;

  //-----------------------------------------------------------
  sg_EnumFormatEtc.RowCount  := gi_EnumFormatEtc.FixedRows + 1;
  sg_EnumFormatEtc.FixedRows := gi_EnumFormatEtc.FixedRows;
  sg_EnumFormatEtc.ColCount  := gi_EnumFormatEtc.TotalColCount;
  sg_EnumFormatEtc.FixedCols := gi_EnumFormatEtc.FixedCols;

  For ColNum := 0 to sg_EnumFormatEtc.ColCount - 1 do
  Begin
    sg_EnumFormatEtc.ColWidths[ColNum] := gci_EnumFormatEtc[ColNum].Width;
    sg_EnumFormatEtc.Cells[ColNum,0] := gci_EnumFormatEtc[ColNum].H0;
    If gi_EnumFormatEtc.FixedRows >= 2 then
      sg_EnumFormatEtc.Cells[ColNum,1] := gci_EnumFormatEtc[ColNum].H1;
  end;
end;

//-------------------------------------------------
procedure Tfm_Main.OnOLEDrop;
//-------------------------------------------------
Var
  ColNum, RowNum: integer;
  FmtNum: integer;
  I: integer;
  S: string;
  pStuff: PChar;
  DDAI : TDropDemoAppInterface;
Const
  Stuff: array[0..50] of char = 'This is a test to see how much stuff there is';
Begin

  // Clear existing contents
  sg_EnumFormatEtc.RowCount := 3;
  For ColNum := 1 to gi_EnumFormatEtc.TotalColCount-1 do
    sg_EnumFormatEtc.Cells[ColNum,2] := '';

  DDAI := FDropAppInterface;
  sg_EnumFormatEtc.RowCount := DDAI.FormatCount + 2;

  For FmtNum := 0 to DDAI.FormatCount -1 do
  Begin
    RowNum := FmtNum + 2;
    sg_EnumFormatEtc.Cells[0,RowNum] := IntToStr(FmtNum);

    I := DDAI.DropRecord[FmtNum].FormatEtc.cfFormat;
    S := IntToStr(I); // default
    sg_EnumFormatEtc.Cells[1,RowNum] := S;
    sg_EnumFormatEtc.Cells[2,RowNum] := cfFormatCaption(I);

    If Assigned(DDAI.DropRecord[FmtNum].FormatEtc.ptd) then S := 'Assigned' else S := 'None';
    sg_EnumFormatEtc.Cells[3,RowNum] := S;

    sg_EnumFormatEtc.Cells[4,RowNum] := dwAspectCaption(DDAI.DropRecord[FmtNum].FormatEtc.dwAspect);

    sg_EnumFormatEtc.Cells[5,RowNum] := IntToStr(DDAI.DropRecord[FmtNum].FormatEtc.lindex);
    sg_EnumFormatEtc.Cells[6,RowNum] := tymedCaption(DDAI.DropRecord[FmtNum].FormatEtc.tymed);

    sg_EnumFormatEtc.Cells[7,RowNum] := IntToStr(DDAI.DropRecord[FmtNum].datasize);
  end;

  mo_Data.Lines.Clear;

end;

end.
